@@ -8,20 +8,27 @@ Created on Sun Oct 31 21:57:14 2021
 
 """ 
 laser_tag_env - a simple grid environment for 1 vs 1 laser tag 
+
+Players can perform an shot and a movement each turn which are limited to either:
+up, down, left or right. A player loses a life if they are shot by their opponent,
+when either player has no lives left the game is over.
+
 """
 
 import numpy as np
 import pygame
+import os
 
-from SimpleRL.envs.base import environment_base 
-from SimpleRL.utils.laser_tag_utils import generate_scenario, shortest_path
+from SimpleRL.envs import environment_base 
+from SimpleRL.utils import generate_scenario, shortest_path
+from SimpleRL.utils import init_video, save_frames, create_video
 
 class laser_tag_env(environment_base):
-
-    # TODO: remove inefficiency in the code (i.e. repeated expressions, improve speed)
-    # TODO: remove matplotlib dependency
     
-    def __init__(self, render=False, seed=None, action_mode="default", enemy_mode="default", difficulty="hard", lives=1):
+    # TODO: is it really necessary to have multi and single action space -> I don't think so
+    # TODO: remove inefficiency in the code (i.e. repeated expressions, improve speed)
+    
+    def __init__(self, render=False, seed=None, action_mode="default", enemy_mode="default", difficulty="hard", lives=1, render_mode="default"):
         
         # Get assertion errors
         
@@ -43,7 +50,26 @@ class laser_tag_env(environment_base):
             + "please select one of the following {} ".format(valid_difficulty)
         assert difficulty in valid_difficulty, difficulty_error
         
+        # Ensure the render_mode for the environment is valid
+        valid_render = ["default", "video"]
+        render_error = "render_mode is not valid for this environment, " \
+            + "please select one of the following {} ".format(valid_render)
+        assert render_mode in valid_render, render_error
+        
+        # Display the input settings 
+        print('\nLaser Tag Settings')
+        print('--------------------')
+        print('Render: {}'.format(render))        
+        print('Seed: {}'.format(seed))
+        print('Action Mode: {}'.format(action_mode))
+        print('Enemy Mode: {}'.format(enemy_mode))
+        print('Difficulty: {}'.format(difficulty))
+        print('Lives: {}'.format(lives))
+        print('Render Mode: {}'.format(render_mode))   
+        print('--------------------')
+        
         self.render = render 
+        self.render_mode = render_mode
         self.seed = seed 
         self.enemy_mode = enemy_mode
         self.action_mode = action_mode
@@ -73,6 +99,7 @@ class laser_tag_env(environment_base):
         
         # Set environmetal parameters
         np.random.seed(self.seed)  
+        self.environment_name = 'Laser_tag'
         self.shot_range = 5 # how many squares will a bullet travel?
         self.grid_size = 8
         self.positive_reward = +1
@@ -81,7 +108,6 @@ class laser_tag_env(environment_base):
         self.enemy_param = 2
         self.terrain_param = 3
         self.empty_param = 0
-        self.display_pause = 0.25
         
         # Initialise the environment
         self.bullet_path = None # for displaying the bullets path
@@ -93,12 +119,21 @@ class laser_tag_env(environment_base):
         # Intialise the display
         if self.render: 
             
+            # Check if there is an available display
+            try: os.environ["DISPLAY"]
+            
+            # Configure a dummy display
+            except: os.environ["SDL_VIDEODRIVER"] = "dummy"
+            
+            if self.render_mode == 'video':
+                self.frame_count, self.image_folder, self.video_folder = init_video(environment_name=self.environment_name)
+            
             # set the screen dimensions
             self.window_width = 400
             self.window_height = 400
             
             # get the screen
-            self.init_display() 
+            self._init_display() 
             
             # set the fps
             self.fps = 5
@@ -134,34 +169,37 @@ class laser_tag_env(environment_base):
         self.check_discrete_input(player_action)  
         
         # update the grid according to the player move
-        reward, done, info = self.update_grid(action=player_action)
+        reward, done, info = self._update_grid(action=player_action)
         
         # display the map
         if self.render:
-            self.display()
+            self._display()
           
         # check if the episode has terminated
         if not done:
             
             # get the computer action
             if self.enemy_mode == "default":
-                computer_action = self.get_computer_action()                  
-                reward, done, info = self.update_grid(action=computer_action)  
+                computer_action = self._get_computer_action()                  
+                reward, done, info = self._update_grid(action=computer_action)  
                 
                 # display the map
                 if self.render:
-                    self.display()                
+                    self._display()                
             
             # get the action of another network
             elif self.enemy_mode == "adversarial":       
                 pass
+               
+        if done and self.render: 
+            self._close_display()         
                  
         return self.grid_map.flatten(), reward, done, info
     
     def sample_discrete_action(self):
         return np.random.randint(self.action_num - 1, size=self.action_dim)
     
-    def update_grid(self, action):
+    def _update_grid(self, action):
         
         # 25 actions --------        
         #      NO|LM|UM|RM|DM
@@ -202,7 +240,7 @@ class laser_tag_env(environment_base):
             # get the grid value of the move           
             final_player_pos = player_pos + chosen_move   
                         
-            valid_move = self.is_move_valid(final_player_pos, self.grid_map)
+            valid_move = self._is_move_valid(final_player_pos, self.grid_map)
             
             # Update the grid if the move is valid
             if valid_move:
@@ -221,7 +259,7 @@ class laser_tag_env(environment_base):
             chosen_move = move_direction[shot_action[0] - 1, :]
             
             # get the outcome of the shot
-            reward, done, info = self.get_shot_trajectory(chosen_move, player_pos, self.grid_map, self.current_player)
+            reward, done, info = self._get_shot_trajectory(chosen_move, player_pos, self.grid_map, self.current_player)
             
             # if the player has been shot remove them
             if done:
@@ -253,7 +291,7 @@ class laser_tag_env(environment_base):
                 
         return reward, done, info    
     
-    def get_shot_trajectory(self, chosen_move, current_player_pos, grid_map, current_player):
+    def _get_shot_trajectory(self, chosen_move, current_player_pos, grid_map, current_player):
                 
         # get the player value
         if current_player == self.player_param:
@@ -299,7 +337,7 @@ class laser_tag_env(environment_base):
             
         return reward, done, info
     
-    def is_move_valid(self, final_player_pos, grid_map):   
+    def _is_move_valid(self, final_player_pos, grid_map):   
         
         # assign the new player position
         row, col = final_player_pos          
@@ -313,7 +351,7 @@ class laser_tag_env(environment_base):
                 
         return False     
     
-    def get_computer_action(self):
+    def _get_computer_action(self):
         
         # select a random action in the case the agent difficulty is set
         random_num = np.random.uniform(0, 1, 1)        
@@ -339,7 +377,7 @@ class laser_tag_env(environment_base):
         for shot_direction in range(directions.shape[0]):
             
             #  check if the game could conclude with a shot            
-            _, done, _ = self.get_shot_trajectory(directions[shot_direction, :], computer_pos, self.grid_map, self.current_player)
+            _, done, _ = self._get_shot_trajectory(directions[shot_direction, :], computer_pos, self.grid_map, self.current_player)
             
             # select the concluding action
             if done: 
@@ -365,7 +403,7 @@ class laser_tag_env(environment_base):
             final_computer_pos =  computer_pos + chosen_move
             
             # Check if the move is valid
-            valid_move = self.is_move_valid(final_computer_pos, self.grid_map)
+            valid_move = self._is_move_valid(final_computer_pos, self.grid_map)
             
             #  check if the game could conclude with a move-shot  
             if valid_move:
@@ -378,7 +416,7 @@ class laser_tag_env(environment_base):
                 temp_grid_map[computer_pos[0], computer_pos[1]] = 0   
                 
                 for shot_direction in range(directions.shape[0]):
-                    _, done, _ = self.get_shot_trajectory(directions[shot_direction, :], final_computer_pos, temp_grid_map, self.current_player)
+                    _, done, _ = self._get_shot_trajectory(directions[shot_direction, :], final_computer_pos, temp_grid_map, self.current_player)
                     
                     # select the concluding action
                     if done: 
@@ -425,7 +463,7 @@ class laser_tag_env(environment_base):
                 final_player_pos =  player_pos + play_chosen_move
                 
                 # Check if the move is valid
-                valid_player_move = self.is_move_valid(final_player_pos, temp_grid_map)
+                valid_player_move = self._is_move_valid(final_player_pos, temp_grid_map)
                 
                 if valid_player_move:
                     
@@ -438,7 +476,7 @@ class laser_tag_env(environment_base):
                     for player_shot_direction in range(directions.shape[0]):
                         
                         #  check if the game could conclude with a shot            
-                        _, done, _ = self.get_shot_trajectory(directions[player_shot_direction, :], final_player_pos, player_temp_grid_map, temp_player)
+                        _, done, _ = self._get_shot_trajectory(directions[player_shot_direction, :], final_player_pos, player_temp_grid_map, temp_player)
                         
                         # mark this computer move as unsafes
                         if done:              
@@ -503,7 +541,7 @@ class laser_tag_env(environment_base):
             return np.array([move_action, 0], dtype=np.int32)
 
             
-    def init_display(self):
+    def _init_display(self):
         
         # quit any previous games
         pygame.display.quit()
@@ -511,14 +549,16 @@ class laser_tag_env(environment_base):
         # initialise pygame
         pygame.init()    
         
+        pygame.display.set_caption("Laser Tag Environment")
+        
         # initialise the clock
         self.clock = pygame.time.Clock()
         
         # create the screen
-        self.screen = pygame.display.set_mode([self.window_height, self.window_width])
+        self.screen = pygame.display.set_mode([self.window_width, self.window_height])
         
     
-    def display(self):   
+    def _display(self):   
         
         # quit the game
         for event in pygame.event.get():
@@ -577,6 +617,10 @@ class laser_tag_env(environment_base):
                 # draw the square
                 pygame.draw.rect(self.screen, colour, rect)    
                 self.screen.blit(text_surface, text_rect)
+        
+        # save frames to the folder
+        if self.render_mode == "video":            
+            self.frame_count = save_frames(screen=self.screen, image_folder=self.image_folder, frame_count=self.frame_count)
           
         # update the display
         pygame.display.update()
@@ -584,9 +628,16 @@ class laser_tag_env(environment_base):
         # update the frame rate
         self.clock.tick(self.fps)
         
-    def close_display(self):
+    def _close_display(self):
+        
+        # shut the display window
         pygame.display.quit()
         
+        # create a video
+        if self.render_mode == 'video':    
+            create_video(image_folder=self.image_folder, video_folder=self.video_folder, fps=self.fps)
+        
+        
 if __name__ == "__main__": 
         
     seed_range = 10
@@ -638,10 +689,7 @@ if __name__ == "__main__":
                     print('Seed {} - Player {} wins'.format(seed, info["outcome"]))
                     
                 state = next_state
-                counter += 1                
-        
-        # close the display
-        env.close_display()
+                counter += 1
         
     print('Player won {}/{}'.format(total_reward, seed_range))
     
